#include <iostream>
#include "semaphore.h"
#include "ThreadPool.h"


struct ThreadData {
  ThreadPool * owner;
  sem_t sem;
  StartRoutine start;
  void * data;
  pthread_t pt;
};


void *threadStart(void *t) {
  ThreadData * td=(ThreadData*)t;
  
  while (true) {
    sem_wait(&td->sem);
    if (td->start == 0)
      return 0 ;
    td->start(td->data);
    td->owner->notifyReady(td);
  }
  
}

void ThreadPool::start(StartRoutine  startroutine, void * param) {
  lock();
  ThreadData * td;
  if (available.empty()) {
    td = new ThreadData();
    td->owner = this;
    sem_init(&td->sem, 0, 0);
    pthread_create(&td->pt, 0, &threadStart, td);
    size++;
    std::cerr <<"Creating a new thread, total "<<size<<std::endl;
  }
  else {
    td = available.front();
    available.pop_front();
    std::cerr <<"Removing from pool, size: "<<available.size()<<"/"<<size<<std::endl;
  }
  unlock();
  td->start = startroutine;
  td->data = param;
  sem_post(&td->sem);
}

void ThreadPool::notifyReady(ThreadData *td) {
  lock();
  available.push_back(td);
  std::cerr <<"Returning to pool, size: "<<available.size()<<"/"<<size<<std::endl;
  unlock();
}
