/** deps.h
// compiler, c lib, operating system dependencies...
*/
//
// The contents of this file are subject to
//	the OpenGo Public License Version 2.0 (the "License"); you may not
//	use this file except in compliance with the License. You may obtain
//	a copy of the License at http://www.inventivity.com/OpenGo
//
// Software distributed under the License is distributed on an "AS IS" basis,
//	WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//	for the specific language governing rights and limitations under
//	the License.
// 
// Copyright:
//	Jeffrey Greenberg	copyright (c) 1996-2002
//	
// Contributor(s):
//
//==========================================================================

#ifndef INC_DEPS
#define INC_DEPS	1



// --------------------------------------------------------------
// Pick your compiler.  DOS/ANSI by default

// #define UNIX


// --------------------------------------------------------------

#ifdef UNIX
	// #warning deps.h Compiling for UNIX!
#elif WIN32
	#pragma message ("deps.h Compiling for Win32!")
#else
	#pragma message ("deps.h Compiling for DOS: default compiler")
#endif


// --------------------------------------------------------------
//	HEADERS


#include <stdlib.h>

#include "basetype.h"
#include "fatal.h"

// vc4:
//#include <stl.h>

#ifdef UNIX
	#include <stream.h>
	#include <sstream>
	#include <strings.h>
	#include <iostream.h>
	#include <iomanip.h>
	#include <stdio.h>
	#include <fstream.h>
#else
	// Not UNIX
	#include <iostream>
	#include <iomanip>
	#include <fstream>
	#include <conio.h>
//	#include <strstrea.h>
#endif	// UNIX


// Portable headers
#include <ctype.h>
#include <math.h>
#include <float.h>
#include <time.h>
#include <string.h>
#include <assert.h>


#ifdef WIN32
	// for NT:
	#include <limits.h>
#else
	// Not NT:
	#include <values.h>
#endif

// vc5:
using namespace std;
#include <list>
#include <vector>
#include <deque>
#include <algorithm>

// --------------------------------------------------------------
//	C RUNTIME FUNCTIONS RECONCILED BY OS AND COMPILER SUPPORT


#ifdef UNIX
// AIX!
	#define strcmpi strcasecmp
	#define stricmp strcasecmp
#endif	// UNIX


#ifndef TRUE
	#define TRUE 1
	#define FALSE 0
#endif	// TRUE


//////////  Random number functions cause portability problems.  Resolve here.
// Define	rand_0to1()  (random float from 0 to 1)
//			rnd_0toNMinus1(x) (random integer from 0 to x-1)
//			rnd_FullRange(x) (random integer)
//			rnd_init() (initialize with a time based seed)						
#ifdef UNIX
// ========================================================
// UNIX stuff
// ========================================================
	#define INT32 int


	//! convert string to lowercase
	inline void strlwr( char *str)
		{
			for( ;*str; str++ )
				*str = tolower(*str);
		}

	// This "extern" may be necessary for your compiler, or it may not.  Comment out if not.
//	extern "C" double random();
//	extern "C" void srandom(int);
//	extern "C" double lrand48();
//	extern "C" void lrand48(long);

	//
	// rand stuff
	//


	//! initialize random number generator using seed
	inline void rnd_init( unsigned long seed )
		{
			srandom( seed );
		}

	//! return in range 0 to less than 1.0
	inline double rand_0to1()
		{
			unsigned long limit = (unsigned long)-1;
			return (double)random()/(double)limit;
		}

	//! return in range 0-limit-1
	inline int rnd_0toNMinus1( int limit )
		{
			if( limit <= 0 )
				return 0;
			int n = ( random() % limit );
			return (n<0?0-n:n);
		}

	//! return full int range
	inline int rnd_FullRange()
                {
		  int i;
                  while ((i=random())==0)
			 ;
		  return i;
		}


#else
// ========================================================
// DOS/ANSI stuff
// ========================================================
// JG: find the compiler pragma or define that could be used
// to detect the compiler...
//#ifndef INT32	// defined in microsoft vc6 but not earlier
//	#define INT32 long
//#endif

	//
	// rand stuff
	//


	//! rand nun ranging between 0 - 1 inclusive.
	// #define rand_0to1() (((float)rand())/RAND_MAX)
	inline float rand_0to1()
	{
		return ((float)rand())/(float)RAND_MAX;
	}

	//! return full int range
	inline unsigned int rnd_uFullRange()	// unsigned has larger magnitude
	{
		return (unsigned int)( (rand()<<1) & 0xfff1) | (rand() & 0x0001);
	}

    inline int rnd_FullRange()				// signed!!
    {
		return rand();
	}


	//! return rand num between 0 and limit-1
	inline int rnd_0toNMinus1(int limit)
	{
		if( limit <= 0 )
			return 0;
		int n = rnd_FullRange() % limit;
		return (n<0?0-n:n);
	}

	//! initialize random number generator using seed
	inline void rnd_init( unsigned long seed )
	{
		srand( seed );
	}


#endif	// UNIX



#endif	// INC_DEPS
