//! iobase.h
//!	std io routines (in an os independent way)
//
// The contents of this file are subject to
//	the OpenGo Public License Version 2.0 (the "License"); you may not
//	use this file except in compliance with the License. You may obtain
//	a copy of the License at http://www.inventivity.com/OpenGo
//
// Software distributed under the License is distributed on an "AS IS" basis,
//	WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//	for the specific language governing rights and limitations under
//	the License.
// 
// Copyright:
//	Jeffrey Greenberg	copyright (c) 1996-2001
//	
// Contributor(s):
//	Jeffrey Greenberg	v1		1996 orig
//	Jeffrey Greenberg	v2		1997
//	Jeffrey Greenberg	v3		1999
//	Jeffrey Greenberg	v4		2000
//	Jeffrey Greenberg	v4.2	2000
//	Jeffrey Greenberg	v5.0	2001
//
//==========================================================================


#ifndef	INC_IOBASE
#define	INC_IOBASE	1
#include "basetype.h"

//! IOBase: abstract application I/O
/**
// IOBase: abstract expression of information
// This is should supplied to a set of classes for them to do I/O
//	in a consistent fashion.  It is especially useful for encapsulating
//	I/O between different GUIs and console interface system.
//	The interface is currently printf style. It might be of interest
//	to extend it to support the iostream interface.
// Example of use:
//	For a Microsoft Windows MDI system, one IOBase could be used for
//	non-window related IO... like for the issues in loading DLLs on
//	menu actions, and then a different IOBase could be created for
//	each window to control i/o within that window.  In the latter
//	case, IOBase would be specialized with routines to support the
//	the Status line, to support a particular view, the title bar, etc.
// Theory:
//	The assumption is that data can be typed to go to a particular sort of
//	display.  This typing is captured by the member functions of IOBase.
//	Those routines can then be specialized to point to the particular
//	GUI display widget to which they are best suited or spec'd for.  The
//	problem with this approach is that the member functions would have
//	to change as the GUI spec changes or the data functionality changes.
//	In the code below, the data functionality at least is done through
//	standard C types ala printf, ameliorating the data problem, though
//	it is neither flexible nor typesafe.
//	
//	An alternative approach is to use different abstract display classes
//	like "StatusBar" and "PersonView" etc.  These classes can then be
//	specialized for a particular GUI.  For a simple console based system,
//	these classes might be implemented as printf()s to stdout and stderr.
//	For a complex GUI, IO would go to a real widget.
*/


class IOBase
{
//	public:
//		IOBase();
//		virtual ~IOBase();

	public:

			// output for developers only.
			//	fatal: express fatal string then exit.
			//	error: express error string
			//	log: express diagnostic string
			//	log: (debug) express diagnostic string in extra debug mode
		virtual void Fatal( char *fmt, ... ) = 0;
		virtual void Error( char *fmt, ... ) = 0;
		virtual void Log( char *fmt, ... ) = 0;
		virtual void Log( BOOL bDebug, char *fmt, ... ) = 0;

			// output for users and developers.
			//	each item is logged as well.
			//  info: express info user might be interested in.
			//  info: express info user might be interested in.
		virtual void Info( char *fmt, ... ) = 0;
		virtual void Prob( char *fmt, ... ) = 0;
};

#endif // INC_IOBASE

