//! move.h
//!	A move is a Piece paired with an MoveType...ie move, pass, quit, etc.
//
// The contents of this file are subject to
//	the OpenGo Public License Version 2.0 (the "License"); you may not
//	use this file except in compliance with the License. You may obtain
//	a copy of the License at http://www.inventivity.com/OpenGo
//
// Software distributed under the License is distributed on an "AS IS" basis,
//	WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//	for the specific language governing rights and limitations under
//	the License.
// 
// Copyright:
//	Jeffrey Greenberg	copyright (c) 1996-2002
// 
// Contributor(s):
//
//==========================================================================


#ifndef INC_MOVE
#define INC_MOVE

#include "basetype.h"
#include "deps.h"
#include "piece.h"

// **************************

typedef U8BIT MoveType;

static const MoveType	MT_None		= 0;
static const MoveType	MT_Move		= 1;
static const MoveType	MT_Pass		= 2;
static const MoveType	MT_Resign	= 3;
static const MoveType	MT_Quit		= 4;
static const MoveType	MT_TakeBack	= 5;



//!	A move is a piece with an action...ie move, pass, quit, etc.
class Move : public Piece
{
	public:
					Move( const MoveType mt, const PieceColor pc, const Coord row, const Coord col );
					Move( const Move &m );
					~Move();
					Move();

	public:
		MoveType	GetMoveType() const
						{
							return _mt;
						};

		void	SetMoveType( MoveType mt )
					{
						_mt = mt;
					};

		void	Set( MoveType mt, PieceColor pc, Coord row, Coord col )
					{
						_mt = mt;
						SetColor( pc );
						SetCoords( row, col );
					};

		void	SetTakeBack( int amt )
					{
						_mt = MT_TakeBack;
						// note coercion could truncate takeback
						// 	for big chunks; but, for most takeback needs
						// 	it'll suffice.
						Assert( amt > 0 );
						SetCoords( 0, (Coord) amt );
					};

		int		GetTakeBack() const
					{
						Assert( _mt == MT_TakeBack);
						Assert( GetCol() > 0 );
						return GetCol();
					};

		//! Get Movetype as string
		char *	GetMoveTypeString( char *buf, int bsize) const
						{
							*buf = 0;
							if( bsize > 6 )
							{
								sprintf( buf, "%s ",
									( _mt==MT_Move ? "Move" :
										( _mt==MT_Pass ? "Pass" :
											( _mt==MT_Resign ? "Resign" :
												( _mt==MT_Quit ? "Quit" :
													"!type!") ) ) ) );
							}
							return buf;
						};

		//! Get entire move as string
		char *	GetString( char *buf, int bsize) const
						{
							*buf = 0;
							GetMoveTypeString( buf, bsize );
							int len = strlen(buf);
							bsize -= len;
							Piece::GetString( &buf[len], bsize );
							return buf;
						};

	private:
		MoveType	_mt;
};

#endif	// INC_MOVE
