// piece.h
//	A piece is a point, Pt, paired with a PieceColor.
//
// The contents of this file are subject to
//	the OpenGo Public License Version 2.0 (the "License"); you may not
//	use this file except in compliance with the License. You may obtain
//	a copy of the License at http://www.inventivity.com/OpenGo
//
// Software distributed under the License is distributed on an "AS IS" basis,
//	WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//	for the specific language governing rights and limitations under
//	the License.
// 
// Copyright:
//	Jeffrey Greenberg	copyright (c) 1996-2002
// 
// Contributor(s):
//
//==========================================================================


#ifndef INC_PIECE
#define INC_PIECE

#include "basetype.h"
#include "deps.h"
#include "pt.h"
#include <string.h>	// for strlen()

typedef U8BIT PieceColor;

const PieceColor	PC_None			= 0;	//! empty
const PieceColor	PC_Black		= 1;
const PieceColor	PC_White		= 2;
const PieceColor	PC_CountMark	= 128;	//! mark used to count empty points...

ostream & operator << ( ostream & os, const PieceColor pc );

class Piece : public Pt
{
	public:
					Piece( const PieceColor pc, const Coord row, const Coord col);
					Piece();
					~Piece();
//					Piece( Piece &p);
					Piece(const Piece &p);

	public:
		PieceColor	GetColor( void ) const
							{ return (_pc & ~PC_CountMark); };

		void			SetColor( PieceColor pc )
							{ _pc = pc; };

		PieceColor	GetOpponentColor() const
				  			{
								switch( _pc & (~PC_CountMark) )
								{
								case PC_White: return PC_Black;
								case PC_Black: return PC_White;
								default: Assert(0); return PC_None;	//shouldn't be called with a non-color
								}
							};

		BOOL		IsEmpty() const
				  			{
								return ( (_pc & (~PC_CountMark)) == 0);
							};

		BOOL		IsNotColor( PieceColor pc) const
				  			{
								return ( (_pc & (~PC_CountMark)) != pc );
							};

		void		GetCoords( Coord *row, Coord *col ) const
							{ *row = GetRow(); *col = GetCol(); }

		void		SetCoords( Coord row, Coord col )
							{ SetRow(row); SetCol(col);};

		void		ClearMark()
							{ _pc &= ~PC_CountMark; };

		void		SetMark()
							{ _pc |= PC_CountMark; };

		BOOL		IsMarked() const
							{ return _pc & PC_CountMark; };

						// return true if got marked
		int			MarkIfEmptyAndNotMarked()
							{
								// if empty:
								if( (_pc & ~PC_CountMark) == PC_None )
								{
									int bWasSet = _pc & PC_CountMark;
									_pc |= PC_CountMark;	// set it now.
									return !bWasSet;
								}
								return 0;	// not blank, so skip
							};

		char *	GetColorString(char *buf, int bsize) const
							{
								*buf = 0;
								if( bsize > 8 )
									sprintf( buf, "%s ",
										( _pc==PC_Black?"Black":
											(_pc==PC_White?"White":"unknown") ) );
								return buf;
							}

		char *	GetString(char *buf, int bsize) const
							{
								*buf = 0;
								if( bsize )
									GetColorString( buf, bsize );
								int len = strlen(buf);
								bsize -= len;
								Pt::GetString( &buf[len], bsize );
								return buf;
							}
	private:
		PieceColor	_pc;
};

#endif	//INC_PIECE
