//! pt.h
//!	representation of a point on a board.
//
// The contents of this file are subject to
//	the OpenGo Public License Version 2.0 (the "License"); you may not
//	use this file except in compliance with the License. You may obtain
//	a copy of the License at http://www.inventivity.com/OpenGo
//
// Software distributed under the License is distributed on an "AS IS" basis,
//	WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//	for the specific language governing rights and limitations under
//	the License.
// 
// Copyright:
//	Jeffrey Greenberg	copyright (c) 1996-2002
// 
// Contributor(s):
//	Don Wagner			v3
//  	Referee's reported move now matches the board view.
//
//==========================================================================

#ifndef INC_PT
#define INC_PT

#include "basetype.h"
#include "deps.h"


#define BOARDDISP_ONEBASED	1	//! Display: Enable coordinates starting from 1.
#define BOARDDISP_SKIPI		1	//! Display: Enable skipping the "i" coord.


typedef S8BIT Coord;	//! important that it is signed...

class Pt
{	
	public:
					Pt( Coord row, Coord col);
//					Pt( Pt &rPt );
					Pt( const Pt &rPt );
					Pt();				//! uninitialized.
					~Pt();
		Coord		GetCol() const {
						return _col; };
		Coord		GetRow() const {
						return _row; };
		void		GetRowCol( Coord *pRow, Coord *pCol) const
						{
							*pRow = _row; *pCol = _col;
						}
		void		SetCol( Coord col )		{ _col = col; };
		void		SetRow( Coord row )		{ _row = row; };
		BOOL		IsInRange( int rowlo, int rowhi, int collo, int colhi ) const
						{
							if(	(_col >= collo)
							&&	(_col < colhi)
							&& (_row >= rowlo)
							&&	(_row < rowhi) )
								return( TRUE );
							return( FALSE );
						};
		Pt	&		operator+=( const Pt &p )
						{
							_row += p._row;
							_col += p._col;
							return *this;
						};
		Pt	&		operator+( Pt &p ) const
						{
							p._row += _row;
							p._col += _col;
							return p;
						};

		char *		GetString(char *buf, int bsize) const
						{
							*buf = 0;
							char col_ch = 'a' + _col;
							#ifdef BOARDDISP_SKIPI
								if( col_ch >= 'i' )
									++col_ch;
							#endif

							int irow = _row;
							#ifdef BOARDDISP_ONEBASED
								++irow;
							#endif

							sprintf( buf, "%c%d", col_ch, irow );
							return buf;
						};

	private:
		Coord		_row;
		Coord		_col;
};

#endif	// INC_PT
