/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is gojax, a Go server.                                       *
 *                                                                   *
 * Copyright 2005                                                    *
 * by Matthieu Nottale.                                              *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 2             *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */  

#include <errno.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "server.h"


TcpAsyncServer * __server=0;



void onReceive(bSocket * s, void * buff, unsigned long len) {
  __server->command(s, (char *) buff);
}

void onError(bSocket * bSock,unsigned long Error) {
  if (errno != EAGAIN && errno != EINTR)
    __server->disconnect(bSock);
}



void * bsloop(void * param) {
  ((bSocket*)param)->EnterReceiveLoop(onReceive, onError, true);
  return 0;
}

void onConnect (bSocket * Waiting,bSocket * Connected) {
  static const char * greet = "ASYNCGO 0.1\n";
  Connected->Send(greet, strlen(greet));
  

  
  __server->pool.start(bsloop, Connected);
}




void  TcpAsyncServer::saveBoard(Game * cg) {
  char name[256];
  sprintf(name, "%s.sgf", cg->name.c_str());
  writesgf(cg->tree.root, name);
  //show_sgf_tree(cg->tree.root);
  if (cg->metadataChanged) {
    sprintf(name, "%s.bd", cg->name.c_str());
    FILE * f = fopen(name, "w");
    if (!f) {
      fprintf(stderr,"Failed to open board metadata file %s for writting\n", name);
      return;
    }
    fprintf(f, "%d\n%s\n%s\n%s\n", (cg->visible?1:0), cg->gamePassword.c_str(), cg->password[0].c_str(), cg->password[1].c_str());
    cg->metadataChanged = false;
    fclose(f);
  }
}

int gofilter(const struct dirent *d) {
  return (strstr(d->d_name,".sgf") !=0);
}

void TcpAsyncServer::loadBoards() {
  struct dirent **namelist;
  int n;
  
  n = scandir(".", &namelist, &gofilter, &alphasort);
  if (n < 0)
    perror("scandir");
  else {
    while(n--) {
      loadBoard(namelist[n]->d_name);
      free(namelist[n]);
    }
    free(namelist);
  }
}

void TcpAsyncServer::loadBoard(char * name) {
  printf("attempting to load %s\n",name);
  Game g;
  g.tree.root = 0;
  sgftree_readfile(&g.tree, name);
  //play board
  SGFNode * cnode= g.tree.root;
  int sz;
  if (sgfGetIntProperty(cnode, "SZ",&sz) ==0) {
    printf("no size property in file %s\n",name);
    return;
  }

  Board *b = new Board(true, true, sz, &io);

  cnode = cnode->child;
  int color = 0;
  int np=0;
  while (cnode) {
    char * move;
    char prop[2]; prop[1]=' ';
    prop[0]= (color==0)?'B':'W';
    int r = sgfGetCharProperty(cnode, prop, &move);
    if (r==0) {
      printf("unexpected node, dumping\n");
      show_sgf_properties(cnode);
      cnode = cnode->child;
      continue;
    }
    char cy=move[0]-'a',cx=move[1]-'a';
    
    // printf("move %d %d %d\n", color+1, cx, cy);
    if (cx<0 || cy<0 ||  cx>=sz || cy>=sz) {
      Move m(MT_Pass, color+1, -1,-1);
      np++;
      if (b->SetMove(&m)!=1) {
	printf("board refused move %d %d\n",(int)cx, (int)cy);
	return;
      }
    }
    else {
      Move m(MT_Move, color+1, cx,cy);
      np = 0;
       if (b->SetMove(&m)!=1) {
	printf("board refused move number %d:%d %d\n",b->GetMoveCount(), (int)cx, (int)cy);
	return;
       }
    }
    color = 1-color;
    cnode = cnode->child;
  }
  
  g.board = b;
  char m[1024];
  strcpy(m, name);
  char *p = strstr(m,".");
  if (p) *p=0;
  g.name = m;
  g.size = sz;  
  g.nPass = np;
  g.requestPending = g.requestDenied = g.requestGranted = 0;
  //loading metadatos
  strcat(m, ".bd");
  FILE * f = fopen(m, "r");
  if (!f) {
    fprintf(stderr,"Failed to open board metadata file %s for reading\n", m);
  }
  else {
    int v;
    m[0]=0;
    fgets(m,1000,f);
    if (strlen(m) && m[strlen(m)-1]=='\n')
      m[strlen(m)-1]=0;
    sscanf(m, "%d", &v);
    g.visible = v;
    m[0]=0;
    fgets(m,1000,f);
    if (strlen(m) && m[strlen(m)-1]=='\n')
      m[strlen(m)-1]=0;
    g.gamePassword = m;
    m[0]=0;
     fgets(m,1000,f);
    if (strlen(m) && m[strlen(m)-1]=='\n')
      m[strlen(m)-1]=0;
    g.password[0] = m;
    m[0]=0;
    fgets(m,1000,f);
    if (strlen(m) && m[strlen(m)-1]=='\n')
      m[strlen(m)-1]=0;
    g.password[1] = m;
    fclose(f);
  }

  
  g.metadataChanged = false;

  g.visible = true;
  lock();
  games.push_back(g);
  unlock();
  printf("loaded %d moves from %s\n",b->GetMoveCount(), name);
}


TcpAsyncServer::TcpAsyncServer(int port)
  : lSocket(false, true) {
  __server = this;
  loadBoards();
 lSocket.CreateWaitLoop(port, onConnect, onError);
}

void TcpAsyncServer::processRequest(Game * cg) {
  if (cg->requestGranted & REQUEST_BACK) {
    if (cg->board->GetMoveCount() <2)
      return; 
    //take back two moves
    cg->board->TakeBackMove();
    cg->board->TakeBackMove();
    while (sgftreeForward(&cg->tree))
      ;
    SGFNode *n = cg->tree.lastnode;
    cg->tree.lastnode = n->parent->parent;
    n->parent->child = 0;
    sgfFreeNode(n);
    sgfFreeNode(cg->tree.lastnode->child);
    cg->tree.lastnode->child=0;
    cg->requestPending = 0;
  }
}


void TcpAsyncServer::command(bSocket *s,char * cmd) {
  fprintf(stderr,"got command %s \n",cmd);
  char msg[1024];
  istringstream is(cmd);
  string c;
  is >> c;
  int player;
  lock();
  Game * cg = find(s, &player);
  if (cg)
    cg->lock();
  unlock();
  if (c == "LIST") {
    //use a buffer because we don't want to be stuck in a Send in a locked state.
    ostringstream os;
    lock();
    for (list<Game>::iterator it=games.begin(); it != games.end();it++) {
      if (! it->visible)
	continue;
      os <<  it->name<<" "<<it->size<<" "<< it->board->GetMoveCount()<<" ";
    }
    unlock();
    s->Send(os.str().c_str()); 
    s->Send("\n");
    if (cg) cg->unlock(); //this line is not stupid, don't remove it or face a dreadful death, whatever dreadful realy means.
    return;
  }
  
  if (c=="BIND") {
    string name; int color=0;
    string pass;
    string res="unknown error";
    is >> name >> color >> pass;
    if  (bind(name, color, s, pass, res))
      s->Send("OK\n");
    else {
      s->Send("FAIL ");
      s->Send(res.c_str());
      s->Send("\n");
    }
    if (cg) cg->unlock();
    return;
  }

   if (c=="JOIN") {
    string name; int color=0;
    string pass, pass2;
    string res;
      is >> name >> color >> pass >>pass2;
    if  (join(name, color, pass, pass2,res)) {
      bind(name, color, s, pass2,res);
      s->Send("OK\n");
    }
    else {
      s->Send("FAIL");
      s->Send(res.c_str());
      s->Send("\n");
    }
    if (cg) cg->unlock();
    return;
  }

  if (c=="STAT") {
    if (!cg) {
      Game g;
      string pass;
      is >> g.name;
      is >> pass;
      lock();
      list<Game>::iterator it = std::find(games.begin(), games.end(), g);
      if (it != games.end()) {
	cg = &(*it);
	cg->lock();
	unlock();
	if (!cg->visible && cg->gamePassword != pass) {
	  s->Send("FAIL wrong password\n");
	  cg->unlock();
	  return;
	}
      }
      else
	unlock();
    }
    if (!cg)
      s->Send("FAIL no game\n");
    else {
      int pw, pb;
      char mx, my;
      
      cg->board->GetCapturedCount(&pb, &pw);
      Move m;
      vector<Piece> inu;
      cg->board->GetNthMove(&m, &inu,0);
      m.GetCoords(&mx, &my);
      sprintf(msg, "OK %d %d %d %d %d %d %c %d %d %d\n", 
	      cg->size,
	      cg->board->GetMoveCount(),
	      (m.GetMoveType()==MT_Move)?my:-1,
	      (m.GetMoveType()==MT_Move)?mx:-1,
	      pb,
	      pw,
	      (cg->nPass>=2)?'o':'r',
	      cg->requestPending,
      	      cg->requestGranted,
	      cg->requestDenied);
      s->Send(msg, strlen(msg));
      
    }
    if (cg) cg->unlock();
    return;
  }

   if (c=="REQUEST") {
     if (!cg)
       s->Send("FAIL no game\n");
     else {
       if ( (player+cg->board->GetMoveCount())%2)
	 s->Send("FAIL not your turn\n");
       else {
	 int i=1;
	 is >>i;
	 if (cg->requestPending & i) {
	   s->Send("FAIL allready pending\n");
	 }
	 else {
	   cg->requestPending |= i;
	   cg->requestDenied &= (~i);
	   cg->requestGranted &= (~i);
	   s->Send("OK\n");
	 }
       }
     }
     if (cg) cg->unlock();
     return;
   }

   if (c=="REQUESTCANCEL") {
     if (!cg)
       s->Send("FAIL no game\n");
     else {
       if ( (player+cg->board->GetMoveCount())%2)
	 s->Send("FAIL not your turn\n");
       else {
	 int i=1;
	 is >>i;
	 if (cg->requestPending & i != i) {
	   s->Send("FAIL no such request\n");
	 }
	 else { 
	 cg->requestPending &= (~i);
	 s->Send("OK\n");
	 }
       }
     }
     if (cg) cg->unlock();
     return;
   }

   if (c=="REQUESTGRANT") {
     if (!cg)
       s->Send("FAIL no game\n");
     else {
       if ( (player+cg->board->GetMoveCount()+1)%2)
	 s->Send("FAIL only your opponent can do this\n");
       else {
	 int i=1;
	 is >>i;
	  if (cg->requestPending & i != i) {
	   s->Send("FAIL no such request\n");
	 }
	 else { 
	   cg->requestPending &= (~i);
	   cg->requestGranted |= i;
	   s->Send("OK\n");
	   processRequest(cg);
	 }
       }
     }
     if (cg) cg->unlock();
     return;
   }
       
     if (c=="REQUESTDENY") {
     if (!cg)
       s->Send("FAIL no game\n");
     else {
       if ( (player+cg->board->GetMoveCount()+1)%2)
	 s->Send("FAIL only your opponent can do this\n");
       else {
	 int i=1;
	 is >>i;
	  if (cg->requestPending & i != i) {
	   s->Send("FAIL no such request\n");
	 }
	 else { 
	   cg->requestPending &= (~i);
	   cg->requestDenied |= i;
	   s->Send("OK\n");
	 }
       }
     }
     if (cg) cg->unlock();
     return;
   }
   
   
  if (c=="PASS") {
     if (!cg || cg->nPass >=2)
       s->Send("FAIL no game\n");
     else {
       if ( (player+cg->board->GetMoveCount())%2)
	 s->Send("FAIL not your turn\n");
       else if (cg->requestPending)
	 s->Send("Pending requests must be granted or denied first\n");
      
       else {
	 Move m(MT_Pass, player+1, -1,-1);
	 if (cg->board->SetMove(&m)==1) {
	   s->Send("OK\n");
	   cg->nPass++;
	   sgftreeAddPlay(&cg->tree, 2-player, -1, -1);
	   saveBoard(cg);
	   cg->requestPending = cg->requestDenied = cg->requestGranted = 0;
	 }
	 else
	   s->Send("FAIL move refused by board\n");
       }
     }
     if (cg) cg->unlock();
    return;
  }
  
  if (c=="PLAY") {
     if (!cg)
       s->Send("FAIL no game\n");
     else {
       int px, py;
       is >> px >> py;
       if ((px<0) || (py<0) || (px>=cg->size) || (py >= cg->size)) {
	 s->Send("FAIL invalid move\n");
       }
       else
	 if ( (player+cg->board->GetMoveCount())%2)
	   s->Send("FAIL not your turn\n");
	 else if (cg->requestPending)
	     s->Send("Pending requests must be granted or denied first\n");
	 else {
	   Move m(MT_Move, player+1, py,px);
	   if (cg->board->SetMove(&m)==1) {
	     s->Send("OK\n");
	     cg->nPass = 0;
	     sgftreeAddPlay(&cg->tree, 2-player, px, py);
	     saveBoard(cg);
	     cg->requestPending = cg->requestDenied = cg->requestGranted = 0;
	   }
	   else
	     s->Send("FAIL move refused by board\n");
	 }
     }
     if (cg) cg->unlock();
    return;
  }

  if (c=="DEAD") {
    if (!cg)
      s->Send("FAIL no game\n");
    else {
      int px, py;
      is >> px >> py;
      if ((px<0) || (py<0) || (px>=cg->size) || (py >= cg->size)) {
	s->Send("FAIL invalid move\n");
      }
      else {
	Pt p(py, px);
	PieceColor pc;
	cg->board->GetPieceAtPos(&p,&pc);
	if (pc != player+1) 
	  s->Send("FAIL not your color\n");
	else {
	  if (!cg->board->MarkAsDead(&p))
	    s->Send("FAIL invalid move\n");
	  else
	    s->Send("OK\n");	  
	}
      }
    }
    if (cg) cg->unlock();
    return;
  }
  
  if (c=="SGF") {
     if (!cg) {
      Game g;
      string pass;
      is >> g.name;
      is >> pass;
      lock();
      list<Game>::iterator it = std::find(games.begin(), games.end(), g);
      if (it != games.end()) {
	cg = &(*it);
	cg->lock();
	unlock();
	if (!cg->visible && cg->gamePassword != pass) {
	  s->Send("FAIL wrong password\n");
	  if (cg) cg->unlock();
	  return;
	}
      }
      else
	unlock();
    }
    if (!cg) 
      s->Send("FAIL no game\n");
    else {
      char fname[1024];
      sprintf(fname, "%s.sgf", cg->name.c_str());
      struct stat st;
      if (stat(fname, &st) != 0) {
	s->Send("FAIL no sgf file found\n");
      }
      else {
	FILE *f = fopen(fname, "r");
	if (!f)
	  s->Send("FAIL failed to open sgf file\n");
	else {
	  s->Send("OK ");
	  while (fgets(fname, 1024,f)) {
	    if (strlen(fname) && fname[strlen(fname)-1]=='\n')
	      fname[strlen(fname)-1]='%';
	    s->Send(fname);
	  }
	  fclose(f);
	  s->Send("\n");
	}
      }
    }
    if (cg) cg->unlock();
    return;
  }



  
  if (c=="BOARD") {
     if (!cg) {
      Game g;
      string pass;
      is >> g.name;
      is >> pass;
      lock();
      list<Game>::iterator it = std::find(games.begin(), games.end(), g);
      if (it != games.end()) {
	cg = &(*it);
	cg->lock();
	unlock();
	if (!cg->visible && cg->gamePassword != pass) {
	  s->Send("FAIL wrong password\n");
	  if (cg) cg->unlock();
	  return;
	}
      }
      else
	unlock();
    }
    if (!cg) 
      s->Send("FAIL no game\n");
    else {
      Board_OptSpace b;
      unsigned char i;
      cg->board->GetBoard(&b, &i);
      PieceColor c;
      for (int y=0;y<cg->size;y++)
	for (int x=0;x<cg->size;x++) {
	  b.GetColorAtRowCol(y,x,&c);
	  msg[x+y*cg->size] = '0'+c;
	}
      msg[cg->size*cg->size]='\n';
      msg[cg->size*cg->size+1]=0;
      s->Send("OK ");
      s->Send(msg);
    }
    if (cg) cg->unlock();
    return;
  }


   if (c=="SCORE") {
     if (!cg)
       s->Send("FAIL no game\n");
     unsigned int b,w,u;
     bool final = cg->board->_ScoreGame(b,w,u);
     sprintf(msg,"OK %d %d %d %d\n",b,w,u,(final?1:0));
     s->Send(msg);
     if (cg) cg->unlock();
     return;
   }
  if (c=="CREATE") {
    string name; int sz=-1;
    is >> name >> sz;
 
    if (sz < 4) {
       s->Send("FAIL invalid size\n");
       return;
    }
    Game dumb;
    dumb.name = name;
    lock();
    if (std::find (games.begin(), games.end(), dumb) != games.end())
      s->Send("FAIL allready exists\n");
    else {
      string p;
      int i;
      is >> i;
      is >> p;
      createGame(name, sz, i, p);
      s->Send("OK\n");
    }
    unlock();
    if (cg) cg->unlock();
    return;
  }

  
  s->Send("FAIL unknown command\n");
  if (cg) cg->unlock();
}



void TcpAsyncServer::createGame(string name, int size, bool visible, string pass) {
  Game g;
  g.name = name;
  g.size = size;
  g.nPass = 0;
  g.gamePassword = pass;
  g.password[0] = g.password[1] = "";
  g.bound[0] = g.bound[1] = 0;
  g.visible = visible;
  g.requestPending = g.requestDenied = g.requestGranted = 0;
  g.metadataChanged = true;
  sgftreeCreateHeaderNode(&g.tree, size, 0);
  g.board = new Board(false, true, size, &io);
  games.push_back(g);
}

Game * TcpAsyncServer::find(bSocket *s, int * player) {
  for (list<Game>::iterator it = games.begin(); it != games.end(); it++) {
    if (it->bound[0]==s) {
      *player = 0;
      return &(*it);
    }
    if (it->bound[1]==s) {
      *player = 1;
      return &(*it);
    }
  }
  return 0;
}


bool TcpAsyncServer::join(string gameName, int color, string gamepass, string pass, string &err) {
  Game dumb;
   dumb.name = gameName;
   list<Game>::iterator it;
   if ((it=std::find (games.begin(), games.end(), dumb)) == games.end()) {
     err="No game by this name";
     return false;
   }
   if (it->gamePassword != gamepass) {
     err="Wrong game password";
     return false;
   }
   if (it->password[color] != ""  && it->password[color] != pass) {
     err="Wrong user password";
     return false;
   }
   if ( it->gamePassword != "" && pass=="") {
     err="Empty passwords not allowed for games with a password";
     return false;
   }
   it->metadataChanged =  (it->password[color] == pass);
   it->password[color] = pass;
   return true;
}

bool TcpAsyncServer::bind(string gameName, int color, bSocket * socket, string pass, string &err) {
  err="";
  Game dumb;
   dumb.name = gameName;
   list<Game>::iterator it; 
   if ((it=std::find (games.begin(), games.end(), dumb)) == games.end()) {
     err="No game by this name";
     return false;
   }
   //check password
   if (it->password[color]!= "" && (it->password[color] != pass)) {
     err="Wrong user password";
     return false;
   }
   if (it->gamePassword != "" && pass == "") {
     err="Empty passwords not allowed for games with a password";
     return false;
   }
   it->bound[color] = socket;
   return true;

}

void TcpAsyncServer::disconnect(bSocket * source) {
  int p;
  Game * g = find(source, &p);
  if (g) 
    g->bound[p] = 0;
}
