/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is gojax, a Go server.                                       *
 *                                                                   *
 * Copyright 2005                                                    *
 * by Matthieu Nottale.                                              *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 2             *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#ifndef INC_TCPASYNCSERVER
#define INC_TCPASYNCSERVER

#include "bsocket_l.h"
#include "lockable.h"
#include "ThreadPool.h"
#include "board/databd.h"
#include "board/iostd.h"
extern "C" {
#include "sgftree.h"
}
typedef DataBoard Board;



#define REQUEST_BACK 1 // request to go back one move

#define DEFAULT_PORT 2242
class Game : public Lockable {
 public:
  string name;
  int size;
  Board * board;
  bSocket * bound[2]; //sockets bound to both players
  string gamePassword;
  string password[2]; //black and white player passwords
  int requestPending;
  int requestGranted;
  int requestDenied;
  int nPass; //number of successive pass


  bool visible ; // false means this game is hidden and does not appear in lists
  SGFTree tree;
  bool metadataChanged; //true if some metadata (passwords) changed
  bool operator ==(const Game &b) const {return name==b.name;}

  Game() {}
  ~Game() {}
};
class TcpAsyncServer: public Lockable {
  
 public:
  TcpAsyncServer(int port=DEFAULT_PORT);
  //load all .sgf boards found in current dir
  void loadBoards();
  void loadBoard(char * filename);
  void saveBoard(Game * cg);
  //creates a new game with the given rules
  void createGame(string name, int size, bool visible, string pass);
  //called for each command received on a socket
  void command(bSocket * source, char * command);
  void disconnect(bSocket * source);
  //locate game and player bound to this socket
  Game * find(bSocket *, int * player);
  //bind a socket to a game and a player
  bool bind(string gameName, int color, bSocket * socket, string pass, string & errmsg);
  //join a game
  bool join(string gameName, int color, string gamePass, string userPass, string & errmsg);


  ThreadPool pool; //public because we need it from callback function
 private:
  list<Game> games;
  bSocket lSocket; 
  IOStd io;
  void processRequest(Game * cg);
};



#endif
